 
// Play wave file activated by a contact microphone. The mic is stuck to a small sheet of acrylic, and covered with fabric on one side.
 
#include <FatReader.h>
#include <SdReader.h>
#include <avr/pgmspace.h>
#include "WaveUtil.h"
#include "WaveHC.h"

SdReader card;    // This object holds the information for the card
FatVolume vol;    // This holds the information for the partition on the card
FatReader root;   // This holds the information for the filesystem on the card
FatReader f;      // This holds the information for the file we're play

WaveHC wave;      // This is the only wave (audio) object, since we will only play one at a time

const int THRESHOLD = 20;
char* FILENAME = "KITCHEN.WAV";
char* WAVFILES [] = {"BARORGAN.WAV", "BEATBOWL.WAV", "BELLRING.WAV", "CHIMES12.WAV", "CHOPPING.WAV", "CREAKING.WAV",
                  "CUPFILL.WAV", "DINNER.WAV", "DOORBOLT.WAV", "DOOROPEN.WAV", "FIRE.WAV", "FOOTSTEP.WAV", "HOOVES.WAV",
                  "KITBELLS.WAV", "KITCHEN.WAV", "LOCKCLCK.WAV", "PASTRY.WAV", "RINSECUP.WAV", "SHEILA.WAV", "STIRRING.WAV",
                  "TAP.WAV", "WALKLAND.WAV"};
int val;                    // The value from the mic
int STROKE_THRESHOLD = 4;   // Value at which to activate sound
int MAXCOUNT = 35;          // Stop counting at this point. This governs how long the sound will take to stop after you stop stroking

unsigned long previousMillis = 0;
int interval = 1000;
long count;

// this handy function will return the number of bytes currently free in RAM, great for debugging!   
int freeRam(void)
{
  extern int  __bss_end; 
  extern int  *__brkval; 
  int free_memory; 
  if((int)__brkval == 0) {
    free_memory = ((int)&free_memory) - ((int)&__bss_end); 
  }
  else {
    free_memory = ((int)&free_memory) - ((int)__brkval); 
  }
  return free_memory; 
} 

void sdErrorCheck(void)
{
  if (!card.errorCode()) return;
  putstring("\n\rSD I/O error: ");
  Serial.print(card.errorCode(), HEX);
  putstring(", ");
  Serial.println(card.errorData(), HEX);
  while(1);
}

void setup() {
  // initialize serial communication:
  Serial.begin(9600);
  
  putstring("Free RAM: ");        // This can help with debugging, running out of RAM is bad
  Serial.println(freeRam());      // if this is under 150 bytes it may spell trouble!
  
  // Set the output pins for the DAC control. This pins are defined in the library
  setupDACPins();
  
  initSDCard();
  
  // Load wave file. We only need to do this once as we're playing a single sound file
  loadWaveFile(FILENAME);
  //loadRandomFile();
}

void loop()
{

  // Read the value produced by the contact mic
  val = analogRead(A0);
  
  // Is the stroking loud enough to start the sound playing (as opposed to 'noise')? If so, the
  // start counting. This ensures that the sound will continue to play in between strokes.
  if(val > THRESHOLD && count <= MAXCOUNT) {
    count+=10;
  }
  
  // Decrement the counter, but slower than incrementing it, as this will occur in between stroked.
  // What we are after is to keep the sound playing when the surface is continually stroked, but
  // stop shortly after the stroking ceases.
  if (val < STROKE_THRESHOLD) {
    count -= 2;
    if (count < 0) count =0; 
  }
  
  if(count <= STROKE_THRESHOLD) {
    if (wave.isplaying) {
      wave.pause();
    } else {
      wave.seek(0); 
    }
  }
  
  if (count > STROKE_THRESHOLD) {
    if (!wave.isplaying) {
      wave.play();
    } else {
      wave.resume(); 
    }
  }
  
  delay(100);
  //Serial.println(count);
}

void loadWaveFile(char * name) {
  // look in the root directory and open the file
  if (!f.open(root, name)) {
    Serial.print("Couldn't open file "); 
    Serial.println(name); return;
  }
  // OK read the file and turn it into a wave object
  if (!wave.create(f)) {
    Serial.println("Not a valid WAV"); return;
  }  
}

// Randomly select and load one of the wav files. This is called when each file has finished playing
void loadRandomFile() {
  
  int randNum = random(0, 22);
  loadWaveFile(WAVFILES[randNum]);
  
}

void setupDACPins() {
  pinMode(2, OUTPUT);
  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);  
}

void initSDCard() {
  if (!card.init()) {         //play with 8 MHz spi (default faster!)  
    putstring_nl("Card init. failed!");  // Something went wrong, lets print out why
    sdErrorCheck();
    while(1);                            // then 'halt' - do nothing!
  }
  
  // enable optimize read - some cards may timeout. Disable if you're having problems
  card.partialBlockRead(true);
  
  // Now we will look for a FAT partition!
  uint8_t part;
  for (part = 0; part < 5; part++) {     // we have up to 5 slots to look in
    if (vol.init(card, part)) 
      break;                             // we found one, lets bail
  }
  
  if (part == 5) {                       // if we ended up not finding one  :(
    putstring_nl("No valid FAT partition!");
    sdErrorCheck();      // Something went wrong, lets print out why
    while(1);                            // then 'halt' - do nothing!
  }
  
  // Lets tell the user about what we found
  putstring("Using partition ");
  Serial.print(part, DEC);
  putstring(", type is FAT");
  Serial.println(vol.fatType(),DEC);     // FAT16 or FAT32?
  
  // Try to open the root directory
  if (!root.openRoot(vol)) {
    putstring_nl("Can't open root dir!"); // Something went wrong,
    while(1);                             // then 'halt' - do nothing!
  }  
}

long microsecondsToInches(long microseconds)
{
  // According to Parallax's datasheet for the PING))), there are
  // 73.746 microseconds per inch (i.e. sound travels at 1130 feet per
  // second).  This gives the distance travelled by the ping, outbound
  // and return, so we divide by 2 to get the distance of the obstacle.
  // See: http://www.parallax.com/dl/docs/prod/acc/28015-PING-v1.3.pdf
  return microseconds / 74 / 2;
}

long microsecondsToCentimeters(long microseconds)
{
  // The speed of sound is 340 m/s or 29 microseconds per centimeter.
  // The ping travels out and back, so to find the distance of the
  // object we take half of the distance travelled.
  return microseconds / 29 / 2;
}

// Plays a full file from beginning to end with no pause.
void playcomplete(char *name) {
  // call our helper to find and play this name
  playfile(name);
  while (wave.isplaying) {
  // do nothing while its playing
  }
  // now its done playing
}

void playfile(char *name) {
  // see if the wave object is currently doing something
  if (wave.isplaying) {// already playing something, so stop it!
    wave.stop(); // stop it
  }
  // look in the root directory and open the file
  if (!f.open(root, name)) {
    putstring("Couldn't open file "); Serial.print(name); return;
  }
  // OK read the file and turn it into a wave object
  if (!wave.create(f)) {
    putstring_nl("Not a valid WAV"); return;
  }
  
  // ok time to play! start playback
  wave.play();
}
